package com.codegeist.fisheye.domain.query;

import static org.junit.Assert.*;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import com.codegeist.fisheye.domain.query.Query.AttributeType;
import com.codegeist.fisheye.domain.query.Query.GroupByType;
import com.codegeist.fisheye.domain.query.Query.OrderByType;
import com.codegeist.fisheye.domain.query.Query.ReturnValue;


public class QueryTest {

	private Query query;

	@Before
	public void setUp() throws Exception {
		query = new Query();
	}
	
	@Test
	public void testEmptyQuery() throws Exception {
		String eyeQl = query.generateEyeQl();
		Assert.assertEquals("select revisions", eyeQl);
	}
	
	@Test
	public void testWithFrom() throws Exception {
		query.setFromDirectory("/test");
		String eyeQl = query.generateEyeQl();
		assertEquals("select revisions from /test", eyeQl);
	}
	
	@Test(expected=IllegalArgumentException.class)
	public void testAddNullReturn() throws Exception {
		query.addReturnValue(null);
	}
	
	@Test
	public void testReturn() throws Exception {
		query.addReturnValue(new ReturnValue(AttributeType.AUTHOR));
		String eyeQl = query.generateEyeQl();
		assertEquals("select revisions return author", eyeQl);
	}
	
	@Test
	public void testReturnWithAs() throws Exception {
		query.addReturnValue(new ReturnValue(AttributeType.AUTHOR, "bling"));
		String eyeQl = query.generateEyeQl();
		assertEquals("select revisions return author as bling", eyeQl);
	}
	
	@Test(expected=IllegalArgumentException.class)
	public void testDuplicateReturn() throws Exception {
		// TODO this is not implemented yet
		query.addReturnValue(new ReturnValue(AttributeType.AUTHOR));
		query.addReturnValue(new ReturnValue(AttributeType.AUTHOR));
	}
	
	@Test
	public void testMultipleReturns() throws Exception {
		query.addReturnValue(new ReturnValue(AttributeType.AUTHOR));
		query.addReturnValue(new ReturnValue(AttributeType.CHANGESET));
		query.addReturnValue(new ReturnValue(AttributeType.COMMENT));
		query.addReturnValue(new ReturnValue(AttributeType.DATE));
		query.addReturnValue(new ReturnValue(AttributeType.IS_ADDED));
		query.addReturnValue(new ReturnValue(AttributeType.IS_BINARY));
		query.addReturnValue(new ReturnValue(AttributeType.IS_COPIED));
		query.addReturnValue(new ReturnValue(AttributeType.IS_DELETED));
		query.addReturnValue(new ReturnValue(AttributeType.IS_MOVED));
		query.addReturnValue(new ReturnValue(AttributeType.LINES_ADDED));
		query.addReturnValue(new ReturnValue(AttributeType.LINES_REMOVED));
		query.addReturnValue(new ReturnValue(AttributeType.PATH));
		query.addReturnValue(new ReturnValue(AttributeType.REVIEWS));
		query.addReturnValue(new ReturnValue(AttributeType.REVISION));
		query.addReturnValue(new ReturnValue(AttributeType.TAGS));
		query.addReturnValue(new ReturnValue(AttributeType.TOTAL_LINES));
		String eyeQl = query.generateEyeQl();
		assertEquals("select revisions return author, csid, comment, date, isAdded, isBinary, isCopied, isDeleted, isMoved, linesAdded, linesRemoved, path, reviews, revision, tags, totalLines", eyeQl);
	}
	
	@Test
	public void testOrderBy() throws Exception {
		query.setOrderBy(OrderByType.DATE);
		String eyeQl = query.generateEyeQl();
		assertEquals("select revisions order by date", eyeQl);
	}
	
	@Test
	public void testGroupByChangeset() throws Exception {
		query.setGroupBy(GroupByType.CHANGESET);
		String eyeQl = query.generateEyeQl();
		assertEquals("select revisions group by changeset", eyeQl);
	}
	
	@Test
	public void testGroupByDirectory() throws Exception {
		query.setGroupBy(GroupByType.DIR);
		String eyeQl = query.generateEyeQl();
		assertEquals("select revisions group by dir", eyeQl);
	}
	
	@Test
	public void testGroupByFile() throws Exception {
		query.setGroupBy(GroupByType.FILE);
		String eyeQl = query.generateEyeQl();
		assertEquals("select revisions group by file", eyeQl);
	}
	
	@Test
	public void testWithSingleRestriction() throws Exception {
		query.addRestriction(new PathRestriction("/trunk/**", false));
		String eyeQl = query.generateEyeQl();
		assertEquals("select revisions where path like /trunk/**", eyeQl);
	}
	
	@Test
	public void testWithMultipleRestrictions() throws Exception {
		query.addRestriction(new PathRestriction("/trunk/**", false));
		query.addRestriction(new AuthorRestriction("brockj"));
		
		String eyeQl = query.generateEyeQl();
		assertEquals("select revisions where path like /trunk/** and author = brockj", eyeQl);
	}
}
