package com.codegeist.fisheye.domain;

/**
 * <pre>
 * struct PathInfo {
 *  String name;    // (REST: attribute)
 *  boolean isFile; // (REST: attribute)
 *  boolean isDir; // (REST: attribute)
 *  boolean isHeadDeleted; // (REST: attribute)
 * }
 * </pre>
 * 
 * @author Brock Janiczak
 */
public class PathInfo {
	private final String name;
	private final boolean isFile;
	private final boolean isDirectory;
	private final boolean isDeleted;
	private final String relativeTo;
	
	public PathInfo(String relativeTo, String name, boolean isFile, boolean isDirectory, boolean isDeleted) {
		this.relativeTo = relativeTo;
		this.name = name;
		this.isFile = isFile;
		this.isDirectory = isDirectory;
		this.isDeleted = isDeleted;
	}

	public String getAbsolutePath() {
		if (relativeTo.endsWith("/")) {
			return relativeTo + name;
		}
		return relativeTo + "/" + name; 
	}
	
	public String getName() {
		return name;
	}

	public boolean isFile() {
		return isFile;
	}

	public boolean isDirectory() {
		return isDirectory;
	}

	public boolean isDeleted() {
		return isDeleted;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (isDeleted ? 1231 : 1237);
		result = prime * result + (isDirectory ? 1231 : 1237);
		result = prime * result + (isFile ? 1231 : 1237);
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		PathInfo other = (PathInfo) obj;
		if (isDeleted != other.isDeleted)
			return false;
		if (isDirectory != other.isDirectory)
			return false;
		if (isFile != other.isFile)
			return false;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}
	
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append(name);
		if (isDirectory) {
			builder.append("/");
		}
		
		if (isDeleted) {
			builder.append(" <deleted>");
		}
		
		return builder.toString();
	}
	
}
