package com.codegeist.fisheye.domain;

public class Version implements Comparable<Version>{
	public static final Version UNKNOWN = new Version(-1, -1, -1);
	
	private int major;
	private int minor;
	private int patch;
	
	Version(int major, int minor, int patch) {
		this.major = major;
		this.minor = minor;
		this.patch = patch;
	}
	
	public int getMajor() {
		return major;
	}
	
	public int getMinor() {
		return minor;
	}
	
	public int getPatch() {
		return patch;
	}
	
	public int compareTo(Version that) {
		if (this.major != that.major) {
			return this.major - that.major;
		}
		
		if (this.minor != that.minor) {
			return this.minor - that.minor;
		}
		
		if (this.patch != that.patch) {
			return this.patch - that.patch;
		}
		
		return 0;
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + major;
		result = prime * result + minor;
		result = prime * result + patch;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Version other = (Version) obj;
		if (major != other.major)
			return false;
		if (minor != other.minor)
			return false;
		if (patch != other.patch)
			return false;
		return true;
	}

	@Override
	public String toString() {
		if (this == Version.UNKNOWN) {
			return "<unknown>";
		}
		
		StringBuilder sb = new StringBuilder();
		sb.append(major);
		sb.append(".");
		sb.append(minor);
		sb.append(".");
		sb.append(patch);
		
		return sb.toString();
	}
	

	public static Version fromString(String versionString) {
		if (versionString == null || versionString.trim().length() == 0) {
			return Version.UNKNOWN;
		}
		
		String[] components = versionString.split("\\.");
		int major = 0;
		int minor = 0;
		int patch = 0;
		
		if (components.length > 0) {
			major = Integer.parseInt(components[0]);
		}
		
		if (components.length > 1) {
			minor = Integer.parseInt(components[1]);
		}
		
		if (components.length > 2) {
			patch = Integer.parseInt(components[2]);
		}
		
		return new Version(major, minor, patch);
		
	}
	
}
