package com.codegeist.fisheye.internal.service;

import java.util.List;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.codegeist.fisheye.domain.PathHistory;
import com.codegeist.fisheye.domain.Revision;

public class PathHistoryHandler extends DefaultHandler implements Handler<PathHistory> {
	private StringBuilder buffer = new StringBuilder();
	private PathHistory result;
	
	private Handler<List<Revision>> revisionsHandler;
	private boolean inRevisions;
	
	
	@Override
	public void startDocument() throws SAXException {
		inRevisions = false;
		revisionsHandler = new ArrayHandler<Revision>("revision", new RevisionHandler());
	}

	@Override
	public void startElement(String uri, String localName, String name, Attributes attributes) throws SAXException {
		if (inRevisions) {
			revisionsHandler.startElement(uri, localName, name, attributes);
		} else {
			if ("revisions".equals(localName)) {
				inRevisions = true;
				revisionsHandler.startDocument();
			}
		}
		buffer.setLength(0);
	}
	
	@Override
	public void endElement(String uri, String localName, String name) throws SAXException {
		if (inRevisions) {
			if ("revisions".equals(localName)) {
				inRevisions = false;
				revisionsHandler.endDocument();
			} else {
				revisionsHandler.endElement(uri, localName, name);
			}
		}

		buffer.setLength(0);
	}
	
	
	@Override
	public void endDocument() throws SAXException {
		List<Revision> revisions = revisionsHandler.getResult();
		result = new PathHistory(revisions.toArray(new Revision[revisions.size()]));
	}
	
	@Override
	public void characters(char[] ch, int start, int length) throws SAXException {
		if (inRevisions) {
			revisionsHandler.characters(ch, start, length);
		} else {
			buffer.append(ch, start, length);
		}
	}
	
	public PathHistory getResult() {
		return result;
	}
}
