package com.codegeist.fisheye.internal.ui.search;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.search.ui.ISearchQuery;
import org.eclipse.search.ui.ISearchResult;
import org.eclipse.search.ui.ISearchResultListener;
import org.eclipse.search.ui.SearchResultEvent;

import com.codegeist.fisheye.domain.RevisionKey;
import com.codegeist.fisheye.domain.query.IQueryMatchListener;
import com.codegeist.fisheye.domain.query.Query.ReturnValue;

public class FisheyeSearchResult implements ISearchResult, IQueryMatchListener {
	private List<ISearchResultListener> listeners;
	private final FisheyeQuery query;
	private List<QueryMatch> matches = new ArrayList<QueryMatch>();
	
	public FisheyeSearchResult(FisheyeQuery query) {
		this.query = query;
		listeners = new ArrayList<ISearchResultListener>();
	}
	
	public void addListener(ISearchResultListener l) {
		synchronized (listeners) {
			listeners.add(l);
		}
	}
	
	public void removeListener(ISearchResultListener l) {
		synchronized (listeners) {
			listeners.remove(l);
		}
	}
	
	public ImageDescriptor getImageDescriptor() {
		return null;
	}

	public String getLabel() {
		return query.getLabel() + " - " + matches.size() + " matches";
	}

	public ISearchQuery getQuery() {
		return query;
	}

	public String getTooltip() {
		return getLabel();
	}

	public List<QueryMatch> getMatches() {
		return matches;
	}
	
	public void matchData(ReturnValue[] columns, Object[] values) {
		QueryMatch match = new QueryMatch();
		for (int i = 0; i < columns.length; i++) {
			ReturnValue column = columns[i];
			Object value = values[i];
			
			switch (column.getAttribute()) {
			case AUTHOR:
				match.setAuthor((String) value);
				break;
			case DATE:
				match.setDate((Date) value);
				break;
			case PATH:
				match.setPath((String) value);
				break;
			case REVISION:
				match.setRevision((String) value);
				break;
			case IS_ADDED:
				match.setNew(true);
				break;
			case LINES_ADDED:
				match.setLinesAdded(((Integer)value).intValue());
				break;
			case LINES_REMOVED:
				match.setLinesRemoved(((Integer)value).intValue());
				break;
			}
		}
		
		synchronized (listeners) {
			SearchResultEvent event = new MatchEvent(this, match);
			for (ISearchResultListener listener : listeners) {
				
				listener.searchResultChanged(event);
			}
		}
		matches.add(match);
	}

	public void matchRevisionKey(RevisionKey revisionKey) {
	}
}
